import {
  collection,
  doc,
  getDoc,
  getDocs,
  addDoc,
  updateDoc,
  deleteDoc,
  query,
  where,
  orderBy,
  limit,
  startAfter,
  DocumentSnapshot,
  serverTimestamp,
  writeBatch,
  QueryConstraint,
  WhereFilterOp,
  Query,
  
} from 'firebase/firestore';
import { db } from './firebase';
import type {
  Subject,
  Topic,
  Content,
  Question,
  Resource,
  UserProgress,
  Achievement,
  AdminUser,
  ReviewStatus,
  DifficultyLevel,
  AgeRange,
  Analytics,
  QuestionOption,
} from '@/types/education';
import type { GameProgress } from '../types/game';

// Generic Types
type CollectionName = 
  | 'subjects'
  | 'topics'
  | 'questions'
  | 'questionOptions'
  | 'userProgress'
  | 'achievements'
  | 'adminUsers'
  | 'content'
  | 'resources';

type DocumentType = 
  | Subject
  | Topic
  | Question
  | QuestionOption
  | Content
  | Resource
  | UserProgress
  | Achievement
  | AdminUser;

// Generic CRUD Functions
export async function getDocument<T extends DocumentType>(
  collectionName: CollectionName,
  id: string
): Promise<T | null> {
  const docRef = doc(db, collectionName, id);
  const docSnap = await getDoc(docRef);
  return docSnap.exists() ? { id: docSnap.id, ...docSnap.data() } as T : null;
}

export async function getDocuments<T extends DocumentType>(
  collectionName: CollectionName,
  options?: {
    whereConditions?: [string, WhereFilterOp, unknown][],
    orderByField?: string,
    limitTo?: number,
    startAfterDoc?: DocumentSnapshot,
  }
): Promise<T[]> {
  const collectionRef = collection(db, collectionName);
  let queryRef: Query = collectionRef;

  if (options) {
    const { whereConditions, orderByField, limitTo, startAfterDoc } = options;
    
    const queryConstraints: QueryConstraint[] = [];

    if (whereConditions) {
      whereConditions.forEach(([field, operator, value]) => {
        queryConstraints.push(where(field, operator, value));
      });
    }

    if (orderByField) {
      queryConstraints.push(orderBy(orderByField));
    }

    if (limitTo) {
      queryConstraints.push(limit(limitTo));
    }

    if (startAfterDoc) {
      queryConstraints.push(startAfter(startAfterDoc));
    }

    if (queryConstraints.length > 0) {
      queryRef = query(collectionRef, ...queryConstraints);
    }
  }

  const querySnapshot = await getDocs(queryRef);
  return querySnapshot.docs.map(doc => ({ id: doc.id, ...doc.data() } as T));
}

export async function createDocument<T extends DocumentType>(
  collectionName: CollectionName,
  data: Omit<T, 'id' | 'createdAt' | 'updatedAt'>
): Promise<T> {
  const docRef = await addDoc(collection(db, collectionName), {
    ...data,
    createdAt: serverTimestamp(),
    updatedAt: serverTimestamp(),
  });

  const newDoc = await getDoc(docRef);
  return { id: newDoc.id, ...newDoc.data() } as T;
}

export async function updateDocument<T extends DocumentType>(
  collectionName: CollectionName,
  id: string,
  data: Partial<T>
): Promise<void> {
  const docRef = doc(db, collectionName, id);
  await updateDoc(docRef, {
    ...data,
    updatedAt: serverTimestamp(),
  });
}

export async function deleteDocument(
  collectionName: CollectionName,
  id: string
): Promise<void> {
  await deleteDoc(doc(db, collectionName, id));
}

// Specialized Functions
export async function createQuestion(
  question: Omit<Question, 'id' | 'createdAt' | 'updatedAt'>,
  options?: Omit<QuestionOption, 'id'>[]
): Promise<{ questionId: string; options?: QuestionOption[] }> {
  const batch = writeBatch(db);

  // Create question
  const questionRef = doc(collection(db, COLLECTIONS.QUESTIONS));
  const questionData = {
    ...question,
    createdAt: serverTimestamp(),
    updatedAt: serverTimestamp(),
  };
  
  if (options) {
    // Create with options using batch
    batch.set(questionRef, questionData);
    
    const createdOptions: QuestionOption[] = [];
    for (const option of options) {
      const optionRef = doc(collection(db, 'questionOptions'));
      const optionData = {
        ...option,
        questionId: questionRef.id,
        createdAt: serverTimestamp(),
        updatedAt: serverTimestamp(),
      };
      batch.set(optionRef, optionData);
      createdOptions.push({ id: optionRef.id, ...option } as QuestionOption);
    }
    
    await batch.commit();
    return {
      questionId: questionRef.id,
      options: createdOptions
    };
  } else {
    // Create without options - simple add
    await batch.set(questionRef, questionData);
    await batch.commit();
    return {
      questionId: questionRef.id
    };
  }
}

export async function updateSubjectOrder(subjectOrders: { id: string; order: number }[]): Promise<void> {
  const batch = writeBatch(db);

  subjectOrders.forEach(({ id, order }) => {
    const docRef = doc(db, 'subjects', id);
    batch.update(docRef, { 
      order,
      updatedAt: serverTimestamp(),
    });
  });

  await batch.commit();
}

export async function getQuestionWithOptions(questionId: string): Promise<{
  question: Question;
  options: QuestionOption[];
} | null> {
  const question = await getDocument<Question>('questions', questionId);
  if (!question) return null;

  const options = await getDocuments<QuestionOption>('questionOptions', {
    whereConditions: [['questionId', '==', questionId]],
    orderByField: 'order',
  });

  return {
    question,
    options,
  };
}

export async function getUserProgressInSubject(userId: string, subjectId: string): Promise<UserProgress[]> {
  return getDocuments<UserProgress>('userProgress', {
    whereConditions: [
      ['userId', '==', userId],
      ['subjectId', '==', subjectId],
    ],
    orderByField: 'lastAttemptDate',
  });
}

// Collection names
const COLLECTIONS = {
  SUBJECTS: 'subjects',
  TOPICS: 'topics',
  CONTENT: 'content',
  QUESTIONS: 'questions',
  RESOURCES: 'resources',
  USER_PROGRESS: 'userProgress',
  ACHIEVEMENTS: 'achievements',
} as const;

// Base interface for pagination
interface PaginationParams {
  pageSize?: number;
  lastDoc?: DocumentSnapshot;
  orderByField?: string;
  orderDirection?: 'asc' | 'desc';
}

// Base interface for filters
interface BaseFilters {
  isActive?: boolean;
  difficulty?: DifficultyLevel;
  ageRange?: AgeRange;
  reviewStatus?: ReviewStatus;
}

// Subject Management
export async function createSubject(subject: Omit<Subject, 'id' | 'createdAt' | 'updatedAt'>): Promise<string> {
  const docRef = await addDoc(collection(db, COLLECTIONS.SUBJECTS), {
    ...subject,
    createdAt: serverTimestamp(),
    updatedAt: serverTimestamp(),
  });
  return docRef.id;
}

export async function getSubject(id: string): Promise<Subject | null> {
  const docRef = await getDoc(doc(db, COLLECTIONS.SUBJECTS, id));
  return docRef.exists() ? { id: docRef.id, ...docRef.data() } as Subject : null;
}

export async function updateSubject(id: string, data: Partial<Subject>): Promise<void> {
  await updateDoc(doc(db, COLLECTIONS.SUBJECTS, id), {
    ...data,
    updatedAt: serverTimestamp(),
  });
}

export async function deleteSubject(id: string): Promise<void> {
  await deleteDoc(doc(db, COLLECTIONS.SUBJECTS, id));
}

// Topic Management
export async function createTopic(topic: Omit<Topic, 'id' | 'createdAt' | 'updatedAt'>): Promise<string> {
  const docRef = await addDoc(collection(db, COLLECTIONS.TOPICS), {
    ...topic,
    createdAt: serverTimestamp(),
    updatedAt: serverTimestamp(),
  });
  return docRef.id;
}

export async function getTopic(id: string): Promise<Topic | null> {
  const docRef = await getDoc(doc(db, COLLECTIONS.TOPICS, id));
  return docRef.exists() ? { id: docRef.id, ...docRef.data() } as Topic : null;
}

export async function updateTopic(id: string, data: Partial<Topic>): Promise<void> {
  await updateDoc(doc(db, COLLECTIONS.TOPICS, id), {
    ...data,
    updatedAt: serverTimestamp(),
  });
}

export async function deleteTopic(id: string): Promise<void> {
  await deleteDoc(doc(db, COLLECTIONS.TOPICS, id));
}

// Content Management
export async function createContent(content: Omit<Content, 'id' | 'createdAt' | 'updatedAt'>): Promise<string> {
  const docRef = await addDoc(collection(db, COLLECTIONS.CONTENT), {
    ...content,
    createdAt: serverTimestamp(),
    updatedAt: serverTimestamp(),
  });
  return docRef.id;
}

export async function getContent(id: string): Promise<Content | null> {
  const docRef = await getDoc(doc(db, COLLECTIONS.CONTENT, id));
  return docRef.exists() ? { id: docRef.id, ...docRef.data() } as Content : null;
}

export async function updateContent(id: string, data: Partial<Content>): Promise<void> {
  await updateDoc(doc(db, COLLECTIONS.CONTENT, id), {
    ...data,
    updatedAt: serverTimestamp(),
  });
}

export async function deleteContent(id: string): Promise<void> {
  await deleteDoc(doc(db, COLLECTIONS.CONTENT, id));
}

// Question Management
export async function getQuestion(id: string): Promise<Question | null> {
  const docRef = await getDoc(doc(db, COLLECTIONS.QUESTIONS, id));
  return docRef.exists() ? { id: docRef.id, ...docRef.data() } as Question : null;
}

export async function updateQuestion(id: string, data: Partial<Question>): Promise<void> {
  await updateDoc(doc(db, COLLECTIONS.QUESTIONS, id), {
    ...data,
    updatedAt: serverTimestamp(),
  });
}

export async function deleteQuestion(id: string): Promise<void> {
  await deleteDoc(doc(db, COLLECTIONS.QUESTIONS, id));
}

// Resource Management
export async function createResource(resource: Omit<Resource, 'id' | 'createdAt' | 'updatedAt'>): Promise<string> {
  const docRef = await addDoc(collection(db, COLLECTIONS.RESOURCES), {
    ...resource,
    createdAt: serverTimestamp(),
    updatedAt: serverTimestamp(),
  });
  return docRef.id;
}

export async function getResource(id: string): Promise<Resource | null> {
  const docRef = await getDoc(doc(db, COLLECTIONS.RESOURCES, id));
  return docRef.exists() ? { id: docRef.id, ...docRef.data() } as Resource : null;
}

export async function updateResource(id: string, data: Partial<Resource>): Promise<void> {
  await updateDoc(doc(db, COLLECTIONS.RESOURCES, id), {
    ...data,
    updatedAt: serverTimestamp(),
  });
}

export async function deleteResource(id: string): Promise<void> {
  await deleteDoc(doc(db, COLLECTIONS.RESOURCES, id));
}

// Fetch functions with filters and pagination
interface SubjectFilters extends BaseFilters {
  category?: string;
  gradeLevel?: string;
}

export async function getSubjects(
  filters: SubjectFilters = {},
  pagination: PaginationParams = {}
): Promise<{ items: Subject[]; lastDoc: DocumentSnapshot | null }> {
  const queryConstraints: QueryConstraint[] = [];

  // Add filters
  if (filters.isActive !== undefined) {
    queryConstraints.push(where('isActive', '==', filters.isActive));
  }
  if (filters.category) {
    queryConstraints.push(where('category', '==', filters.category));
  }
  if (filters.gradeLevel) {
    queryConstraints.push(where('gradeLevel', '==', filters.gradeLevel));
  }

  // Add pagination
  if (pagination.orderByField) {
    queryConstraints.push(orderBy(pagination.orderByField, pagination.orderDirection || 'asc'));
  }
  if (pagination.pageSize) {
    queryConstraints.push(limit(pagination.pageSize));
  }
  if (pagination.lastDoc) {
    queryConstraints.push(startAfter(pagination.lastDoc));
  }

  const q = query(collection(db, COLLECTIONS.SUBJECTS), ...queryConstraints);
  const querySnapshot = await getDocs(q);

  return {
    items: querySnapshot.docs.map(doc => ({ id: doc.id, ...doc.data() } as Subject)),
    lastDoc: querySnapshot.docs[querySnapshot.docs.length - 1] || null
  };
}

interface TopicFilters extends BaseFilters {
  subjectId?: string;
}

export async function getTopics(
  filters: TopicFilters = {},
  pagination: PaginationParams = {}
): Promise<{ items: Topic[]; lastDoc: DocumentSnapshot | null }> {
  const constraints: QueryConstraint[] = [];

  // Add filters
  if (filters.isActive !== undefined) {
    constraints.push(where('isActive', '==', filters.isActive));
  }
  if (filters.subjectId) {
    constraints.push(where('subjectId', '==', filters.subjectId));
  }
  if (filters.difficulty) {
    constraints.push(where('difficulty', '==', filters.difficulty));
  }
  if (filters.ageRange) {
    constraints.push(where('ageRange', '==', filters.ageRange));
  }

  // Add ordering
  const orderByField = pagination.orderByField || 'order';
  const orderDirection = pagination.orderDirection || 'asc';
  constraints.push(orderBy(orderByField, orderDirection));

  // Add pagination
  if (pagination.pageSize) {
    constraints.push(limit(pagination.pageSize));
  }
  if (pagination.lastDoc) {
    constraints.push(startAfter(pagination.lastDoc));
  }

  const q = query(collection(db, COLLECTIONS.TOPICS), ...constraints);
  const snapshot = await getDocs(q);

  return {
    items: snapshot.docs.map(doc => ({ id: doc.id, ...doc.data() }) as Topic),
    lastDoc: snapshot.docs[snapshot.docs.length - 1] || null,
  };
}

interface ContentFilters extends BaseFilters {
  topicId?: string;
  subjectId?: string;
  type?: string;
  visibility?: string;
}

export async function getContents(
  filters: ContentFilters = {},
  pagination: PaginationParams = {}
): Promise<{ items: Content[]; lastDoc: DocumentSnapshot | null }> {
  const constraints: QueryConstraint[] = [];

  // Add filters
  if (filters.isActive !== undefined) {
    constraints.push(where('isActive', '==', filters.isActive));
  }
  if (filters.topicId) {
    constraints.push(where('topicId', '==', filters.topicId));
  }
  if (filters.subjectId) {
    constraints.push(where('subjectId', '==', filters.subjectId));
  }
  if (filters.type) {
    constraints.push(where('type', '==', filters.type));
  }
  if (filters.difficulty) {
    constraints.push(where('difficulty', '==', filters.difficulty));
  }
  if (filters.ageRange) {
    constraints.push(where('ageRange', '==', filters.ageRange));
  }
  if (filters.reviewStatus) {
    constraints.push(where('reviewStatus', '==', filters.reviewStatus));
  }
  if (filters.visibility) {
    constraints.push(where('visibility', '==', filters.visibility));
  }

  // Add ordering
  const orderByField = pagination.orderByField || 'order';
  const orderDirection = pagination.orderDirection || 'asc';
  constraints.push(orderBy(orderByField, orderDirection));

  // Add pagination
  if (pagination.pageSize) {
    constraints.push(limit(pagination.pageSize));
  }
  if (pagination.lastDoc) {
    constraints.push(startAfter(pagination.lastDoc));
  }

  const q = query(collection(db, COLLECTIONS.CONTENT), ...constraints);
  const snapshot = await getDocs(q);

  return {
    items: snapshot.docs.map(doc => ({ id: doc.id, ...doc.data() }) as Content),
    lastDoc: snapshot.docs[snapshot.docs.length - 1] || null,
  };
}

interface QuestionFilters extends BaseFilters {
  topicId?: string;
  subjectId?: string;
  type?: string;
  bloomsTaxonomyLevel?: string;
}

export async function getQuestions(
  filters: QuestionFilters = {},
  pagination: PaginationParams = {}
): Promise<{ items: Question[]; lastDoc: DocumentSnapshot | null }> {
  const constraints: QueryConstraint[] = [];

  // Add filters
  if (filters.isActive !== undefined) {
    constraints.push(where('isActive', '==', filters.isActive));
  }
  if (filters.topicId) {
    constraints.push(where('topicId', '==', filters.topicId));
  }
  if (filters.subjectId) {
    constraints.push(where('subjectId', '==', filters.subjectId));
  }
  if (filters.type) {
    constraints.push(where('type', '==', filters.type));
  }
  if (filters.difficulty) {
    constraints.push(where('difficulty', '==', filters.difficulty));
  }
  if (filters.ageRange) {
    constraints.push(where('ageRange', '==', filters.ageRange));
  }
  if (filters.reviewStatus) {
    constraints.push(where('reviewStatus', '==', filters.reviewStatus));
  }
  if (filters.bloomsTaxonomyLevel) {
    constraints.push(where('bloomsTaxonomyLevel', '==', filters.bloomsTaxonomyLevel));
  }

  // Add ordering
  const orderByField = pagination.orderByField || 'createdAt';
  const orderDirection = pagination.orderDirection || 'desc';
  constraints.push(orderBy(orderByField, orderDirection));

  // Add pagination
  if (pagination.pageSize) {
    constraints.push(limit(pagination.pageSize));
  }
  if (pagination.lastDoc) {
    constraints.push(startAfter(pagination.lastDoc));
  }

  const q = query(collection(db, COLLECTIONS.QUESTIONS), ...constraints);
  const snapshot = await getDocs(q);

  return {
    items: snapshot.docs.map(doc => ({ id: doc.id, ...doc.data() }) as Question),
    lastDoc: snapshot.docs[snapshot.docs.length - 1] || null,
  };
} 

// Analytics
export const getAnalytics = async (): Promise<Analytics> => {
  try {
    const [subjects, topics, contents] = await Promise.all([
      getSubjects(),
      getTopics(),
      getContents()
    ]);

    // Mock data for user analytics (replace with actual implementation)
    const analytics: Analytics = {
      totalUsers: 1000,
      activeUsers: 750,
      totalSubjects: subjects.items.length,
      totalTopics: topics.items.length,
      totalContent: contents.items.length,
      completionRate: 85,
      engagementRate: 78
    };

    return analytics;
  } catch (error) {
    console.error('Error getting analytics:', error);
    throw error;
  }
}; 

// Subject Operations
export const addSubject = async (subject: Subject) => {
  try {
    const docRef = await addDoc(collection(db, 'subjects'), {
      ...subject,
      createdAt: serverTimestamp(),
      updatedAt: serverTimestamp()
    });
    return docRef.id;
  } catch (error) {
    console.error('Error adding subject:', error);
    throw error;
  }
};

export const getSimpleSubjects = async () => {
  const { items } = await getSubjects();
  return items;
};

// Question Operations
export const addQuestion = async (question: Question) => {
  try {
    const docRef = await addDoc(collection(db, 'questions'), question);
    return docRef.id;
  } catch (error) {
    console.error('Error adding question:', error);
    throw error;
  }
};

export const getQuestionsBySubject = async (subjectId: string, ageRange: string): Promise<Question[]> => {
  try {
    const q = query(
      collection(db, 'questions'),
      where('subjectId', '==', subjectId),
      where('ageRange', '==', ageRange)
    );
    const querySnapshot = await getDocs(q);
    return querySnapshot.docs.map(doc => ({
      id: doc.id,
      ...doc.data()
    })) as Question[];
  } catch (error) {
    console.error('Error getting questions:', error);
    throw error;
  }
};

// Game Progress Operations
export const saveGameProgress = async (progress: GameProgress): Promise<string> => {
  try {
    const docRef = await addDoc(collection(db, 'gameProgress'), {
      ...progress,
      timestamp: new Date()
    });
    return docRef.id;
  } catch (error) {
    console.error('Error saving game progress:', error);
    throw error;
  }
};

export async function getTopScores(
  subjectId: string,
  limitCount: number = 10
): Promise<GameProgress[]> {
  try {
    const q = query(
      collection(db, 'gameProgress'),
      where('subjectId', '==', subjectId),
      orderBy('score', 'desc'),
      limit(limitCount)
    );
    const querySnapshot = await getDocs(q);
    return querySnapshot.docs.map(doc => {
      const data = doc.data();
      return {
        score: data.score || 0,
        questionsAnswered: data.questionsAnswered || 0,
        correctAnswers: data.correctAnswers || 0,
        timeSpent: data.timeSpent || 0,
        subjectId: data.subjectId || '',
      } as GameProgress;
    });
  } catch (error) {
    console.error('Error getting top scores:', error);
    return [];
  }
} 